# End-to-end Testing

There are 5 processes, so you'll need 5 terminal windows or screen / tmux
sessions. Run the processes in this order:

1. QUIC relay server: `RUST_LOG=quic_demo_relay_server=debug cargo run --bin quic_demo_relay_server`
2. Server-side proxy: `RUST_LOG=quic_demo_end_server=debug cargo run --bin quic_demo_end_server`
3. Client-side proxy: `RUST_LOG=quic_demo_client cargo run --bin quic_demo_client`
4. TCP end server: `nc -l -p 30382`
5. TCP end client: `nc 127.0.0.1 30381`

The netcat processes from steps 1 and 5 should now be connected to each other.

# Testing PTTH itself

The end-to-end testing above is the happy path. Try these sadder cases:

- Swap Steps 2 and 3
- After Step 2, restart the server proxy P4
- After Step 3, restart the client proxy P2
- After Step 5, close P1. P2 and P4 should stay up
- After Step 2, restart the relay server P3
- After Step 3, restart the relay server P3

# Network protocol

For the prototype, all control messages are fixed 4-byte messages at the
start of bidirectional streams.

Unused bytes are always "0".

## QUIC client connection

When P2 or P4 connects to P3 over QUIC, it opens a bi stream and identifies
itself with a 4-byte request. The bytes are:

1. Client type
2. Client ID
3. Unused
4. Unused

The type is "2" or "4" to identify the client as a P2 or a P4. The IDs are
hard-coded as "42" and "43" for now.

P3 responds with these 4 bytes:

1. Status code
2. Unused
3. Unused
4. Unused

The status code is always "20" to mean "OK". If authorization or authentication
had failed, it would be something else, and P3 would close the connection.

## PTTH connection

When P1 connects to P2's TCP listener, P2 opens a bi stream to P3 and sends
this request:

1. "1" to mean "Open a PTTH connection"
2. Server ID, hard-coded as "43" for the only P4 server instance we have
3. Unused
4. Unused

The first 4 bytes are interpreted by P3. If everything is okay, P3 responds
with:

1. "20" to mean "OK".
2. Unused
3. Unused
4. Unused

Then P3 opens a bi stream to P4 and begins relaying from P2 to P4.

P2 reads that response and sends another request, which will be forwarded
to P4:

1. "1" to mean "Open a PTTH connection"
2. Unused (Reserved for choosing port)
3. Unused
4. Unused

P4 parses the last 4 bytes. If everything is okay, P4 (via P3) responds to P2 with:

1. "20" to mean "OK".
2. Unused
3. Unused
4. Unused

And then P4 begins relaying between P2 and P5.

P2 reads the 4 byte response from P3 and the 4 byte response from P4.
If everything is okay, it completes the relaying from P1 to P5.

# Plan

This is a TCP port forwarding system, like SSH has, but better.

We'll name the processes in order from clientest to serverest:

1. TCP end client (e.g. VNC viewer, SSH client)
2. Client-side proxy (Part of PTTH, must run as a desktop app on end user systems)
3. Relay server (Part of ptth_relay)
4. Server-side proxy (Path of ptth_server)
5. TCP end server (e.g. VNC server, SSH server)

At the highest level, creating a connection means:

1. Processes 2, 3, and 4 are running and connected by QUIC.
2. Process 5 is running and listening on its TCP port. 
(e.g. 22 for SSH)
3. Process 2 sets a port mapping to the server. 
(e.g. 2200 on the client is mapped to 22 on the server)
4. Process 1 connects to the client-side port
(e.g. ssh to 2200 on localhost)
5. Processes 2, 3, and 4 forward that connection to Process 5.
Any of the other processes can reject the connection for various reasons.
6. The connection is established, and Processes 2, 3, and 4 relay bytes
for Processes 1 and 5.

What identifies a connection?

1. The client ID, because a connection is never shared between 2 clients.
2. The ephemeral port used by P1, because 2 SSH clients may connect to the same
forwarded port in P2 at once.
3. The server ID, because a connection is never shared between 2 servers.
4. The listen port of P5, for the sake of matching TCP theory, and because
this data is needed. I'm not sure if it's technically part of the identifier.

Because clients can't see each other, each client will store these data
about each of its connections:

1. The ephemeral port used by P1
2. The server ID
3. The listen port for P5

Because servers can't see each other, each server stores these data:

1. P1's port
2. The client ID
3. The listen port for P5

The relay server must store all 4 data for each connection.

Each client and each server has exactly one QUIC connection to the relay
server.

In P2 and P4, each PTTH connection has a TCP bi stream and a QUIC bi stream.

In P3, each PTTH connection has 2 QUIC bi streams.

Assume that some kind of framing exists.

Once a port mapping is established (within P2 only), a connection proceeds
like this:

1. P1 connects to P2
2. P2 starts a QUIC bi stream to P3. This has a tip message of
(P1 port, P4 ID, P5 port).
3. If P2 isn't authorized to make the connection, P3 closes it
4. If P4 isn't connected to P3, the connection fails
5. P3 starts a QUIC bi stream to P4. This has a tip message of
(P1 port, P2 ID, P5 port).
6. If P4 doesn't approve, the connection closes
7. If P4 can't connect to P5, the connection closes
8. P4 connects to P5
9. P4 returns a tip message to P3, equivalent to "200 OK"
10. P3 returns the tip message to P2
11. The connection goes into relaying mode

Here's the big things I'm thinking:

- It's kinda Tor, in that there's 3 middle nodes and onion layers
- It's kinda like HTTP 1, except after the server responds, you turn the
connection into an arbitrary TCP connection. Maybe that's what the CONNECT
verb does?
- I will probably need a control channel some day, but not today

So I don't need a general control stream, which means luckily I don't need
to worry about synchronizing between streams. The tip messages are enough
to get things going.

Would the protocol be cleaner if the tip messages were explicitly layered?
I think so. As long as we aren't end-to-end encrypted (AND WE ARE NOT) this
gives P3, the relay server, a chance to be clever if we ever need it.

So reviewing the above steps:

In Step 2 the tip message says, "Connect me to P4 with ID 'bogus' and forward
this message to them".

Between Steps 2 and 5, P3 re-wraps the inner message. P3 says to P4,
"P2 with ID 'bogus' wants to connect, and forwards this message"

P4 receives that inner tip message in Step 5, and it says "Connect my port 2200
to your port 22."

There's no need for the actual port numbers to be used - It could be tokenized.
But for a prototype, I think it's fine.

Since all parties trust P3, this makes it more clear that P2 and P4 may not
directly trust each other. P3 vouches for the identity of each. The response
from P4 is symmetrical with the request from P2.


